from datetime import datetime
from django import forms


class TwitterAuthForm(forms.Form):
    auth_time = forms.IntegerField(min_value=0, initial=int(datetime.now().strftime('%s')), required=True)
    user_id = forms.IntegerField(min_value=0, required=True)
    oauth_token = forms.CharField(required=True)
    oauth_token_secret = forms.CharField(required=True)
    screen_name = forms.CharField(required=True)



from django.test import TestCase
from django.urls import resolve, reverse
from twitter_oauth_ios.views import AuthView


class TestUrlResolveTests(TestCase):

    def setUp(self):
        self.auth_view = AuthView.as_view()

    def tearDown(self):
        self.auth_view = None

    def test_url_resolves_to_auth_view(self):
        found = resolve(reverse('twitter_oauth_ios:auth_view'))

        self.assertEqual(found.func.view_class, AuthView)



from django.conf import settings
from requests_oauthlib import OAuth1

import requests


def get_display_name(user_id, oauth_token, oauth_token_secret):
    oauth = OAuth1(
        settings.SOCIAL_AUTH_TWITTER_KEY,
        settings.SOCIAL_AUTH_TWITTER_SECRET,
        oauth_token,
        oauth_token_secret
    )

    response = requests.get(
        'https://api.twitter.com/1.1/users/show.json?user_id={}'.format(user_id),
        auth=oauth
    ).json()

    return response['name']



from django.test import TestCase
from twitter_oauth_ios.forms import TwitterAuthForm


class TestTwitterAuthForm(TestCase):

    form_class = TwitterAuthForm

    def setUp(self):
        self.parameter = {
            "auth_time": 1526996947,
            "user_id": 1111111111,
            "oauth_token": "1111111111-this_is_test_oauth_token",
            "oauth_token_secret": "this_is_test_oauth_token_secret",
            "screen_name": "test_screen_name",
        }

    def tearDown(self):
        self.parameter = None

    def test_is_valid_form(self):
        form = self.form_class(self.parameter)

        self.assertTrue(form.is_valid())

    def test_is_invalid_form(self):
        form = self.form_class({})

        self.assertFalse(form.is_valid())



import os


BASE_DIR = os.path.dirname(os.path.dirname(__file__))

DEBUG = True

ADMINS = (
    ('admin', 'admin@localhost')
)

MANAGERS = ADMINS


DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(BASE_DIR, 'test_db.sqlite3'),
    }
}

TIME_ZONE = 'America/Chicago'

LANGUAGE_CODE = 'en-us'
USE_I18N = True
USE_L10N = True

SITE_ID = 1

MEDIA_ROOT = ''

MEDIA_URL = ''

SECRET_KEY = '#($gm0!3&=yy^-3aka=0#y#2b1i$qn51tc$vpmplum1_$az8-='

MIDDLEWARE = (
    'django.middleware.common.CommonMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
)

ROOT_URLCONF = 'tests.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [os.path.join(BASE_DIR, 'templates')],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

INSTALLED_APPS = (
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'social_django',
    'twitter_oauth_ios.apps.TwitterOauthIosConfig',
    'twitter_oauth_ios.tests'
)

AUTHENTICATION_BACKENDS = (
    'django.contrib.auth.backends.ModelBackend',
)

TEST_RUNNER = 'twitter_oauth_ios.tests.test_runner.TestRunner'

STATIC_ROOT = 'staticfiles'
STATIC_URL = '/static/'

SOCIAL_AUTH_TWITTER_KEY = ''
SOCIAL_AUTH_TWITTER_SECRET = ''



from datetime import datetime
from django.core.management import call_command
from django.contrib.auth import get_user_model
from django.test import TestCase
from django.test.client import RequestFactory
from django.urls import reverse
from social_django.models import UserSocialAuth
from twitter_oauth_ios.views import AuthView
from unittest.mock import patch, Mock

import json


class TestAuthView(TestCase):

    endpoint = reverse('twitter_oauth_ios:auth_view')
    user_model = get_user_model()

    def setUp(self):
        self.auth_view = AuthView.as_view()
        self.factory = RequestFactory()
        self.parameter = {
            "user_id": 1111111111,
            "oauth_token": "1111111111-this_is_test_oauth_token",
            "oauth_token_secret": "this_is_test_oauth_token_secret",
            "screen_name": "test_screen_name",
        }

    def tearDown(self):
        self.factory = None
        self.parameter = None
        call_command('flush', interactive=False)

    def test_not_allow_method(self):
        request = self.factory.post(self.endpoint, self.parameter)
        request.content_type = 'application/json'
        response = self.auth_view(request)

        self.assertEqual(response.status_code, 405)

    def test_failure_put(self):
        request = self.factory.put(self.endpoint, {})
        request.content_type = 'application/json'
        response = self.auth_view(request)

        self.assertEqual(json.loads(response.content.decode('utf-8')), {'result': 'error'})
        self.assertEqual(response.status_code, 400)

    def test_success_create_put(self):

        user = self.user_model.objects.create_user(
            username=self.parameter['screen_name'],
            first_name='This is display_name',
            is_active=True
        )
        user.save()

        extra_data = {
            'auth_time': int(datetime.now().strftime('%s')),
            'id': self.parameter['user_id'],
            'access_token': {
                'oauth_token': self.parameter['oauth_token'],
                'oauth_token_secret': self.parameter['oauth_token_secret'],
                'user_id': self.parameter['user_id'],
                'screen_name': self.parameter['screen_name']
            }
        }
        user_social_auth = UserSocialAuth(
            user=user,
            provider='twitter',
            uid=self.parameter['user_id'],
            extra_data=extra_data
        )
        user_social_auth.save()

        with patch('requests.get') as patcher:
            mock_json = Mock()
            mock_json.json.return_value = {'name': 'This is display_name'}
            patcher.return_value = mock_json

            request = self.factory.put(self.endpoint, json.dumps(self.parameter))
            request.content_type = 'application/json'

            response = self.auth_view(request)
            self.assertEqual(json.loads(response.content.decode('utf-8')), {'result': 'success'})
            self.assertEqual(response.status_code, 200)

    def test_success_update_put(self):
        user = self.user_model.objects.create_user(
            username=self.parameter['screen_name'],
            first_name='This is display_name',
            is_active=True
        )
        user.save()

        extra_data = {
            'auth_time': int(datetime.now().strftime('%s')),
            'id': self.parameter['user_id'],
            'access_token': {
                'oauth_token': self.parameter['oauth_token'],
                'oauth_token_secret': self.parameter['oauth_token_secret'],
                'user_id': self.parameter['user_id'],
                'screen_name': self.parameter['screen_name']
            }
        }
        user_social_auth = UserSocialAuth(
            user=user,
            provider='twitter',
            uid=self.parameter['user_id'],
            extra_data=extra_data
        )
        user_social_auth.save()

        with patch('requests.get') as patcher:
            mock_json = Mock()
            mock_json.json.return_value = {'name': 'This is display_name'}
            patcher.return_value = mock_json

            request = self.factory.put(self.endpoint, json.dumps(self.parameter))
            request.content_type = 'application/json'
            response = self.auth_view(request)

        self.assertEqual(json.loads(response.content.decode('utf-8')), {'result': 'success'})
        self.assertEqual(response.status_code, 200)



from datetime import datetime
from django.contrib.auth import get_user_model
from django.http.response import JsonResponse
from django.views import View
from django.views.decorators.csrf import csrf_exempt
from django.utils.decorators import method_decorator
from social_django.models import UserSocialAuth
from .forms import TwitterAuthForm
from .utils import get_display_name

import json


class AuthView(View):
    form_class = TwitterAuthForm
    user_model = get_user_model()

    def put(self, request, *args, **kwargs):
        body = json.loads(request.body.decode('utf-8'))
        body['auth_time'] = int(datetime.now().strftime('%s'))
        form = self.form_class(body)

        if not form.is_valid():
            return JsonResponse({'result': 'error'}, status=400)

        social_auth = UserSocialAuth.get_social_auth(provider='twitter', uid=form.data['user_id'])

        display_name = get_display_name(form.data['user_id'], form.data['oauth_token'], form.data['oauth_token_secret'])

        if social_auth:
            user = social_auth.user
            user.username = form.data['screen_name']
            user.first_name = display_name
            user.save()
            social_auth.user = user

            extra_data = social_auth.extra_data
            extra_data['auth_time'] = form.data['auth_time']
            extra_data['access_token']['screen_name'] = form.data['screen_name']

            social_auth.extra_data = extra_data
            social_auth.save()

            return JsonResponse({'result': 'success'})

        user = self.user_model.objects.create_user(
            username=form.data['screen_name'],
            first_name=display_name,
            is_active=True)
        user.save()

        extra_data = {
            'auth_time': form.data['auth_time'], 'id': form.data['user_id'],
            'access_token': {
                'oauth_token': form.data['oauth_token'],
                'oauth_token_secret': form.data['oauth_token_secret'],
                'user_id': form.data['user_id'],
                'screen_name': form.data['screen_name']
            }
        }
        social_auth = UserSocialAuth(user=user, provider='twitter', uid=form.data['user_id'], extra_data=extra_data)
        social_auth.save()

        return JsonResponse({'result': 'success'})

    @method_decorator(csrf_exempt)
    def dispatch(self, *args, **kwargs):
        return super(AuthView, self).dispatch(*args, **kwargs)



from django.conf import settings
from django.test.runner import DiscoverRunner as TestRunnerBase


class TestRunner(TestRunnerBase):
    def setup_test_environment(self, **kwargs):
        super(TestRunner, self).setup_test_environment(**kwargs)
        settings.TESTING = True

    def teardown_test_environment(self, **kwargs):
        super(TestRunner, self).teardown_test_environment(**kwargs)
        settings.TESTING = True
